;PhysicEngine_3.au3
#include-once

#include <Misc.au3>
#include <Timers.au3>
#include <PhysicEngine_3_Collision.au3>
#include <PhysicEngine_3_Settings.au3>

Func phys_Startup()
	phys_LoadSettings()							
	phys_StartupDraw()							
	OnAutoItExitRegister( "phys_Shutdown" )
EndFunc

Func phys_Shutdown()
	phys_ShutdownDraw() 
	phys_SaveSettings()
EndFunc

Global $g_StartTime = _Timer_Init()
Global $g_TimerDiff_Last = _Timer_Diff($g_StartTime)

Func phys_Frame()
	
	Local $TimerDiff = _Timer_Diff($g_StartTime)
	Local $DeltaT_s = 0.001 * ($TimerDiff - $g_TimerDiff_Last) 	
	$g_TimerDiff_Last = $TimerDiff
	
	If $DeltaT_s > $PHYS_FIX_TIMESTEP_S Then
		Local $SliceTime = $DeltaT_s - $PHYS_FIX_TIMESTEP_S
		$DeltaT_s = $PHYS_FIX_TIMESTEP_S
		WinSetTitle( $g_hWindow, "", $g_Window_Title & " - " & Round( $DeltaT_s * 1000, 0 ) & "ms " & Round( 1/$DeltaT_s, 0 ) & "fps FIX slice: " & Round( $SliceTime * 1000, 0 ) & "ms" )
	Else
		WinSetTitle( $g_hWindow, "", $g_Window_Title & " - " & Round( $DeltaT_s * 1000, 0 ) & "ms " & Round( 1/$DeltaT_s, 0 ) & "fps" )
	EndIf
	
	phys_UserInput()
	
	phys_PolygonsUpdatePoints()
	
	For $i = 0 To $g_NumPolygons - 1 Step 1
		phys_PolygonAddForceCenter( $i, $PHYS_GRAVITY_X, $PHYS_GRAVITY_Y )
	Next	
	
	For $i = 0 To $g_NumCircles - 1 Step 1
		phys_CircleAddForceCenter( $i, $PHYS_GRAVITY_X, $PHYS_GRAVITY_Y )
	Next
	
	phys_PolygonsApplyForces( $DeltaT_s )
	phys_CirclesApplyForces( $DeltaT_s )
	
	phys_Collisions()
	
	phys_PolygonsUpdatePosition( $DeltaT_s, $PHYS_DAMPING_COEFFICIENT, $PHYS_SLEEP_MAX_MOVEMENT_SQ, $PHYS_SLEEP_MAX_ANGLE, $PHYS_SLEEP_TIME )
	phys_CirclesUpdatePosition( $DeltaT_s, $PHYS_DAMPING_COEFFICIENT, $PHYS_SLEEP_MAX_MOVEMENT_SQ, $PHYS_SLEEP_MAX_ANGLE, $PHYS_SLEEP_TIME )
	
	;phys_PolygonsUpdatePoints()
	
	phys_Draw( $DeltaT_s )
	
	;ToolTip( "DeltaT: " & Round( $DeltaT_s * 1000, 0 ) & "ms -> " & Round( 1/$DeltaT_s, 0 ) & "fps" )
EndFunc

Global $g_Selected_Radius[2]
Global $g_Selected_ID = -1
Enum $OBJECT_POLYGON, $OBJECT_CIRCLE
Global Const $NON_SELECTED = -1
Global $g_Selected_Type = 0

Func phys_Draw( $delta_t_s )
		
	Switch $PHYS_DRAW_MODE
		Case $RENDER_NULL
		Case $RENDER_GDIPLUS
			phys_ClearBackbuffer()	
	
			For $i = 0 To $g_NumPolygons - 1 Step 1
				phys_DrawPolygonGDIPlus( $i )
			Next
			
			For $i = 0 To $g_NumCircles - 1 Step 1
				phys_DrawCircle( $i )
			Next	
			
			Local $MousePosition = MouseGetPos()
			
			If $g_Selected_ID <> $NON_SELECTED Then
				If $g_Selected_Type = $OBJECT_POLYGON Then
					Local $Sin = Sin( $g_Polygons_Angle[$g_Selected_ID] )
					Local $Cos = Cos( $g_Polygons_Angle[$g_Selected_ID] )
					
					Local $ApplyPointX = $g_Polygons_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
					Local $ApplyPointY = $g_Polygons_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
					
					phys_DrawCrossGDIPlus($ApplyPointX, $ApplyPointY)
					phys_DrawCrossGDIPlus($g_Polygons_Position_X[$g_Selected_ID], $g_Polygons_Position_Y[$g_Selected_ID])
					
					If _IsPressed( 0x1 ) Then phys_DrawLineGDIPlus($ApplyPointX,$ApplyPointY,$MousePosition[0],$MousePosition[1])
				ElseIf $g_Selected_Type = $OBJECT_CIRCLE Then
					Local $Sin = Sin( $g_Circles_Angle[$g_Selected_ID] )
					Local $Cos = Cos( $g_Circles_Angle[$g_Selected_ID] )
					
					Local $ApplyPointX = $g_Circles_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
					Local $ApplyPointY = $g_Circles_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
					
					phys_DrawCrossGDIPlus($ApplyPointX, $ApplyPointY)
					phys_DrawCrossGDIPlus($g_Circles_Position_X[$g_Selected_ID], $g_Circles_Position_Y[$g_Selected_ID])
					
					If _IsPressed( 0x1 ) Then phys_DrawLineGDIPlus($ApplyPointX,$ApplyPointY,$MousePosition[0],$MousePosition[1])
				EndIf
			EndIf
			
			;_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$g_point_2[0],$g_point_2[1],$g_point_2[0] + $g_mtd_2[0] * 20,$g_point_2[1] + $g_mtd_2[1] * 20, $g_hPen)
			
			phys_DrawCrossGDIPlus( $MousePosition[0], $MousePosition[1] )
			
			phys_DrawLineGDIPlus($depthest_point_x,$depthest_point_y,$depthest_point_x+$penetration_x,$depthest_point_y+$penetration_y)
			
			For $i = 0 To $g_NumCollisionsPoints-1 Step 1
				phys_DrawLineGDIPlus($g_CollisionsPoints_PointX[$i], $g_CollisionsPoints_PointY[$i],$g_CollisionsPoints_PointX[$i]+$g_CollisionsPoints_NormalX[$i]*20, $g_CollisionsPoints_PointY[$i]+$g_CollisionsPoints_NormalY[$i]*20)
			Next
			
			phys_SwapBuffers()	
		Case $RENDER_OPENGL			
			glClearColor( 0.5, 0.6, 1.0, 1.0 )
			glClear($GL_COLOR_BUFFER_BIT)
			
			For $i = 0 To $g_NumPolygons - 1 Step 1
				phys_DrawPolygonOpenGL( $i, $g_Polygons_Points_X[$i], $g_Polygons_Points_Y[$i], $g_Polygons_Points_Translated_X[$i], $g_Polygons_Points_Translated_Y[$i] )
			Next
			
			Local $MousePosition = MouseGetPos()
			
			If $g_Selected_ID <> $NON_SELECTED Then
				If $g_Selected_Type = $OBJECT_POLYGON Then
					Local $Sin = Sin( $g_Polygons_Angle[$g_Selected_ID] )
					Local $Cos = Cos( $g_Polygons_Angle[$g_Selected_ID] )
					
					Local $ApplyPointX = $g_Polygons_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
					Local $ApplyPointY = $g_Polygons_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
					
					phys_DrawCrossOpengl($ApplyPointX, $ApplyPointY)
					phys_DrawCrossOpengl($g_Polygons_Position_X[$g_Selected_ID], $g_Polygons_Position_Y[$g_Selected_ID])
					
					If _IsPressed( 0x1 ) Then phys_DrawLineOpengl($ApplyPointX,$ApplyPointY,$MousePosition[0],$MousePosition[1])
				ElseIf $g_Selected_Type = $OBJECT_CIRCLE Then
					Local $Sin = Sin( $g_Circles_Angle[$g_Selected_ID] )
					Local $Cos = Cos( $g_Circles_Angle[$g_Selected_ID] )
					
					Local $ApplyPointX = $g_Circles_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
					Local $ApplyPointY = $g_Circles_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
					
					phys_DrawCrossOpengl($ApplyPointX, $ApplyPointY)
					phys_DrawCrossOpengl($g_Circles_Position_X[$g_Selected_ID], $g_Circles_Position_Y[$g_Selected_ID])
					
					If _IsPressed( 0x1 ) Then phys_DrawLineOpengl($ApplyPointX,$ApplyPointY,$MousePosition[0],$MousePosition[1])
				EndIf
			EndIf
			
			;_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$g_point_2[0],$g_point_2[1],$g_point_2[0] + $g_mtd_2[0] * 20,$g_point_2[1] + $g_mtd_2[1] * 20, $g_hPen)
			
			phys_DrawCrossOpengl( $MousePosition[0], $MousePosition[1] )
			
			phys_DrawLineOpengl($depthest_point_x,$depthest_point_y,$depthest_point_x+$penetration_x,$depthest_point_y+$penetration_y)
			
			For $i = 0 To $g_NumCollisionsPoints-1 Step 1
				phys_DrawLineOpengl($g_CollisionsPoints_PointX[$i], $g_CollisionsPoints_PointY[$i],$g_CollisionsPoints_PointX[$i]+$g_CollisionsPoints_NormalX[$i]*20, $g_CollisionsPoints_PointY[$i]+$g_CollisionsPoints_NormalY[$i]*20)
			Next
			
			SwapBuffers($g_OpenGL_DC)
		Case Else
			Return False
	EndSwitch
	
EndFunc

Global $penetration_x = 0
Global $penetration_y = 0
Global $penetration_depth = 0 
Global $depthest_point_x = 0
Global $depthest_point_y = 0

Func phys_Collisions()
	_phys_LogCollisionsReset()

	For $i = 0 To $g_NumPolygons - 1 Step 1
		For $j = $i + 1 To $g_NumPolygons - 1 Step 1
			If (($g_Polygons_Mass[$i] <> 0) Or ($g_Polygons_Mass[$j] <> 0) Or ($g_Polygons_Tensor[$i] <> 0) Or ($g_Polygons_Tensor[$j] <> 0)) And (Not $g_Polygons_Sleep[$i] Or Not $g_Polygons_Sleep[$j]) Then
			
				Local $point_x = 0
				Local $point_y = 0
				Local $normal_x = 0
				Local $normal_y = 0
				
				If phys_PolygonsIntersect( $i, $j, $g_Polygons_Points_Translated_X[$i], $g_Polygons_Points_Translated_Y[$i], $g_Polygons_Points_Translated_X[$j], $g_Polygons_Points_Translated_Y[$j], $point_x, $point_y, $normal_x, $normal_y ) Then
					
					;ToolTip("collision polygon-polygon! point: " & $point_x & " " & $point_y & " normal: " & $normal_x & " " & $normal_y)
					_phys_LogCollisionsPoint( $point_x, $point_y, $normal_x, $normal_y )	
					
					phys_ResponseCollisionPolygons( $i, $j, $point_x, $point_y, $normal_x, $normal_y )	
					;phys_ResponseCollisionPolygons( $i, $j, $point_x, $point_y, -$normal_y, $normal_x )
					;phys_ResponseCollisionPolygons( $i, $j, $point_x, $point_y, $normal_y, -$normal_x )
					
					phys_PolygonsPenetrationDepth( $i, $j, $g_Polygons_Points_Translated_X[$i], $g_Polygons_Points_Translated_Y[$i], $g_Polygons_Points_Translated_X[$j], $g_Polygons_Points_Translated_Y[$j], $normal_x, $normal_y, $penetration_x, $penetration_y, $penetration_depth, $depthest_point_x, $depthest_point_y )
					
					If $PHYS_FRICTION Then
						phys_ResponseFrictionPolygon( $i, $point_x, $point_y, -$normal_x, -$normal_y )
						phys_ResponseFrictionPolygon( $j, $point_x, $point_y,  $normal_x,  $normal_y )
					EndIf
					
					$g_Polygons_Sleep[$i] = False
					$g_Polygons_Sleep[$j] = False
					
					If $PHYS_CORRECT_POSITION Then phys_PolygonsFixPositions( $i, $j, $penetration_x, $penetration_y )
				EndIf
			EndIf
		Next
	Next
	
	For $i = 0 To $g_NumPolygons - 1 Step 1
		For $j = 0 To $g_NumCircles - 1 Step 1
			If (($g_Polygons_Mass[$i] <> 0) Or ($g_Circles_Mass[$j] <> 0) Or ($g_Polygons_Tensor[$i] <> 0) Or ($g_Circles_Tensor[$j] <> 0)) And (Not $g_Polygons_Sleep[$i] Or Not $g_Circles_Sleep[$j]) Then
			
				Local $point_x = 0
				Local $point_y = 0
				Local $normal_x = 0
				Local $normal_y = 0
			
				If phys_PolygonCircleIntersect( $i, $j, $point_x, $point_y, $normal_x, $normal_y ) Then
					
					_phys_LogCollisionsPoint( $point_x, $point_y, $normal_x, $normal_y )
					
					phys_ResponseCollisionPolygonCircle( $i, $j, $point_x, $point_y, $normal_x, $normal_y )
					;phys_ResponseCollisionPolygonCircle( $i, $j, $point_x, $point_y, -$normal_y, $normal_x )
					;phys_ResponseCollisionPolygonCircle( $i, $j, $point_x, $point_y, $normal_y, -$normal_x )
					
					phys_PolygonCirclePenetrationDepth( $i, $j, $normal_x, $normal_y, $penetration_x, $penetration_y, $penetration_depth, $depthest_point_x, $depthest_point_y )
					;ToolTip("collision polygon-circle! point: " & $point_x & " " & $point_y & " normal: " & $normal_x & " " & $normal_y & " penetration: " & $penetration_x & " " & $penetration_y)
					If $PHYS_FRICTION Then
						phys_ResponseFrictionPolygon( $i, $point_x, $point_y, -$normal_x, -$normal_y )
						phys_ResponseFrictionCircle( $j, $point_x, $point_y,  $normal_x,  $normal_y )
					EndIf
					
					$g_Polygons_Sleep[$i] = False
					$g_Circles_Sleep[$j] = False
					
					If $PHYS_CORRECT_POSITION Then phys_PolygonCircleFixPositions( $i, $j, $penetration_x, $penetration_y )
				EndIf
			EndIf
		Next
	Next
	
	For $i = 0 To $g_NumCircles - 1 Step 1
		For $j = $i+1 To $g_NumCircles - 1 Step 1
			If (($g_Circles_Mass[$i] <> 0) Or ($g_Circles_Mass[$j] <> 0) Or ($g_Circles_Tensor[$i] <> 0) Or ($g_Circles_Tensor[$j] <> 0)) And (Not $g_Circles_Sleep[$i] Or Not $g_Circles_Sleep[$j]) Then
			
				Local $point_x = 0
				Local $point_y = 0
				Local $normal_x = 0
				Local $normal_y = 0
			
				If phys_CirclesIntersect( $i, $j, $point_x, $point_y, $normal_x, $normal_y ) Then
					
					_phys_LogCollisionsPoint( $point_x, $point_y, $normal_x, $normal_y )
					;ToolTip("collision circle-circle! point: " & $point_x & " " & $point_y & " normal: " & $normal_x & " " & $normal_y)
					phys_ResponseCollisionCircles( $i, $j, $point_x, $point_y, $normal_x, $normal_y )
					;phys_ResponseCollisionCircles( $i, $j, $point_x, $point_y, -$normal_y, $normal_x )
					;phys_ResponseCollisionCircles( $i, $j, $point_x, $point_y, $normal_y, -$normal_x )
					
					phys_CirclesPenetrationDepth( $i, $j, $normal_x, $normal_y, $penetration_x, $penetration_y, $penetration_depth, $depthest_point_x, $depthest_point_y )
					
					If $PHYS_FRICTION Then
						phys_ResponseFrictionCircle( $i, $point_x, $point_y, -$normal_x, -$normal_y )
						phys_ResponseFrictionCircle( $j, $point_x, $point_y,  $normal_x,  $normal_y )
					EndIf
					
					$g_Circles_Sleep[$i] = False
					$g_Circles_Sleep[$j] = False
					
					If $PHYS_CORRECT_POSITION Then phys_CirclesFixPositions( $i, $j, $penetration_x, $penetration_y )
				EndIf
			EndIf
		Next
	Next
EndFunc

Global $g_Mouse_Left_Last = False
Global $g_Key_C_Last = False

Func phys_UserInput()
	If Not $PHYS_DEBUG Then Return 0

	phys_ReadDebugWindow()
	
	Local $MousePosition = MouseGetPos()
	Local $MouseLeft = _IsPressed( 0x1 ) ; left mouse	
	Local $KeyC = _IsPressed( 43 ); c key
	
	If $MouseLeft And Not $g_Mouse_Left_Last Then
		$g_Selected_ID = $NON_SELECTED
		
		Local $Radius
		
		If phys_PolygonFromPoint( $MousePosition[0], $MousePosition[1], $g_Selected_ID ) Then
			$g_Selected_Type = $OBJECT_POLYGON
			$Radius = phys_PolygonGetRadiusPoint( $g_Selected_ID, $MousePosition[0], $MousePosition[1] )
			Local $Sin = Sin( -$g_Polygons_Angle[$g_Selected_ID] )
			Local $Cos = Cos( -$g_Polygons_Angle[$g_Selected_ID] )
						
			$g_Selected_Radius[0] = $Cos * $Radius[0] - $sin * $Radius[1]
			$g_Selected_Radius[1] = $Sin * $Radius[0] + $Cos * $Radius[1]
		ElseIf phys_CircleFromPoint( $MousePosition[0], $MousePosition[1], $g_Selected_ID ) Then
			$g_Selected_Type = $OBJECT_CIRCLE
			$Radius = phys_CircleGetRadiusPoint( $g_Selected_ID, $MousePosition[0], $MousePosition[1] )
			Local $Sin = Sin( -$g_Circles_Angle[$g_Selected_ID] )
			Local $Cos = Cos( -$g_Circles_Angle[$g_Selected_ID] )
						
			$g_Selected_Radius[0] = $Cos * $Radius[0] - $sin * $Radius[1]
			$g_Selected_Radius[1] = $Sin * $Radius[0] + $Cos * $Radius[1]
		EndIf
	EndIf
	
	If $g_Selected_ID <> $NON_SELECTED Then
		If $g_Selected_Type = $OBJECT_POLYGON Then	
			GUICtrlSetData( $g_DebugWindow_Info, phys_PolygonGetInfomations($g_Selected_ID) )
				
			If $MouseLeft Then ; left mouse	 
				Local $Sin = Sin( $g_Polygons_Angle[$g_Selected_ID] )
				Local $Cos = Cos( $g_Polygons_Angle[$g_Selected_ID] )
				
				Local $ApplyPointX = $g_Polygons_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
				Local $ApplyPointY = $g_Polygons_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
				
				Local $ForceX = 10 * ($MousePosition[0] - $ApplyPointX)
				Local $ForceY = 10 * ($MousePosition[1] - $ApplyPointY)
				
				phys_PolygonAddForcePoint( $g_Selected_ID, $ApplyPointX, $ApplyPointY, $ForceX, $ForceY )
			EndIf
			
			If _IsPressed( 0x2 ) Then ; right mouse
				$g_Polygons_Position_X[$g_Selected_ID] = $MousePosition[0]
				$g_Polygons_Position_Y[$g_Selected_ID] = $MousePosition[1]
			EndIf
			
			If _IsPressed( 0x34 ) Then ; r key
				phys_PolygonStop($g_Selected_ID)
			EndIf
			
			If _IsPressed( 41 ) Then ; left key
				$g_Polygons_Angle[$g_Selected_ID] -= $DEG_TO_RAD
			EndIf
			
			If _IsPressed( 44 ) Then ; right key
				$g_Polygons_Angle[$g_Selected_ID] += $DEG_TO_RAD
			EndIf
			
			If Not $g_Key_C_Last And $KeyC Then
				$g_Selected_ID = phys_CopyPolygon( $g_Selected_ID )
			EndIf	

			If _IsPressed( "4A" ) Then phys_PolygonScale( $g_Selected_ID, $g_Polygons_Points_X[$g_Selected_ID], $g_Polygons_Points_Y[$g_Selected_ID], 0.99, 1 )	; j key
			If _IsPressed( "4C" ) Then phys_PolygonScale( $g_Selected_ID, $g_Polygons_Points_X[$g_Selected_ID], $g_Polygons_Points_Y[$g_Selected_ID], 1.01, 1 )	; l key
			If _IsPressed( "49" ) Then phys_PolygonScale( $g_Selected_ID, $g_Polygons_Points_X[$g_Selected_ID], $g_Polygons_Points_Y[$g_Selected_ID], 1, 0.99 )	; i key
			If _IsPressed( "4B" ) Then phys_PolygonScale( $g_Selected_ID, $g_Polygons_Points_X[$g_Selected_ID], $g_Polygons_Points_Y[$g_Selected_ID], 1, 1.01 )	; k key
			
			If _IsPressed( "53" ) Then phys_PolygonSleep( $g_Selected_ID )			
			If _IsPressed( "57" ) Then phys_PolygonWake( $g_Selected_ID )	
			
			If _IsPressed( "45" ) Then
				phys_PolygonDelete( $g_Selected_ID )
				$g_Selected_ID = $NON_SELECTED
			EndIf
		ElseIf $g_Selected_Type = $OBJECT_CIRCLE Then
			GUICtrlSetData( $g_DebugWindow_Info, phys_CircleGetInfomations($g_Selected_ID) )
			
			If $MouseLeft Then ; left mouse	 
				Local $Sin = Sin( $g_Circles_Angle[$g_Selected_ID] )
				Local $Cos = Cos( $g_Circles_Angle[$g_Selected_ID] )
				
				Local $ApplyPointX = $g_Circles_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
				Local $ApplyPointY = $g_Circles_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
				
				Local $ForceX = 10 * ($MousePosition[0] - $ApplyPointX)
				Local $ForceY = 10 * ($MousePosition[1] - $ApplyPointY)
				
				phys_CircleAddForcePoint( $g_Selected_ID, $ApplyPointX, $ApplyPointY, $ForceX, $ForceY )
			EndIf
			
			If _IsPressed( 0x2 ) Then ; right mouse
				$g_Circles_Position_X[$g_Selected_ID] = $MousePosition[0]
				$g_Circles_Position_Y[$g_Selected_ID] = $MousePosition[1]
			EndIf
			
			If _IsPressed( 0x34 ) Then ; r key
				phys_CircleStop($g_Selected_ID)
			EndIf
			If _IsPressed( "53" ) Then phys_CircleSleep( $g_Selected_ID )		
			If _IsPressed( "57" ) Then phys_CircleWake( $g_Selected_ID )
		EndIf
	EndIf
	
	$g_Mouse_Left_Last = $MouseLeft
	$g_Key_C_Last = $KeyC
EndFunc



